/*
 *   3dfb -- a graphical multidimensional filebrowser
 *   written by Tilmann Bitterberg (tilmann@bitterberg.de)
 *   Copyright (C) 1999 by Tilmann Bitterberg
 *   
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *   
 *   Thanks to Max Kliche <max@kliche.org> for support an ideas
 *   Also thanks to the HfT-Stuttgart <http://www.fht-stuttgart.de>
 */
/* include Dateien */

#include "myhdrs.h" 

int get_dir_info(char *dirname, struct dir_info_struct *dir_info) 
{
  struct dirent **namelist;
  struct dirent **subnamelist;
  struct stat     name;
  struct stat     subname;
  int n=0, i=0;
  int curelem=0;
  char buffer[1024];
  char *temp=NULL;
  char argument[256];

  errno=0;
  temp=0;
  dir_info->sub_dirs=0;
  dir_info->non_dirs=0;
  dir_info->sub_info=NULL;
  strcpy(dir_info->dir_name, dirname);
  strcpy(argument, dirname);

  chdir(dirname);
  strcpy(dirname,getcwd(temp, 1024));
  /* dirname is now the full path */
  free(temp);

  if (geteuid() == 0 ){
    fprintf(stderr, "You should not do this as root.\n");
    exit(1);
  } 
  if (errno == EACCES)
  {
    fprintf(stderr, "You don't have permission to read %s !\n",argument );
    fprintf(stderr, "Go away and get a life.\n");
    /*
    dir_info->sub_dirs=-1;
    dir_info->non_dirs=-1;
    errno=0;
    return(1);
    */
  }
  n = scandir(".", &namelist, NULL, alphasort);
  if (n<0) {
    /*This can never? happen*/
    fprintf(stderr, "Uhh?.\n");
    return(1); 
  }
  else
  {
    while(n--)
    {
      stat(namelist[n]->d_name,&name);
      /* S_ISDIR ist das Makro */
      /* erklaert in man 2 stat */
      if(!(S_ISDIR(name.st_mode)))
      {
	/*fprintf(stdout, "%d. Kein Directory! %s\n",n,namelist[n]->d_name);*/
	dir_info->non_dirs++;
      }	
      else
      {   /* Directory */
	/*fprintf(stdout, "%d. Directory! %s\n",n,namelist[n]->d_name);*/
	curelem = ++dir_info->sub_dirs;

	if (!(dir_info->sub_info = (struct sub_dir_info_struct *) \
	      realloc(dir_info->sub_info, \
		sizeof(struct sub_dir_info_struct)*(dir_info->sub_dirs)) ) )
	  exit(1); /* realloc failed */
	
	strcpy(buffer, dirname);
	if (buffer[strlen(buffer)-1] != '/') strcat(buffer,"/");
	strcat(buffer, namelist[n]->d_name);
	strcpy(dir_info->sub_info[curelem-1].sub_dir_name, buffer);
	
	/*printf("working=%s\n",getcwd(NULL, 0));*/
	chdir(namelist[n]->d_name); /* change to ... */
	if (errno == EACCES)
	{
	  /* permission denied, chdir() failed */
	  dir_info->sub_info[curelem-1].sub_dirs=-1;
	  errno=0;
	}
	else /* permission granted */
	{
	  i = scandir(".", &subnamelist, NULL, alphasort);

	  dir_info->sub_info[curelem-1].sub_dirs=0;
	  while(i--)
	  {
	    stat(subnamelist[i]->d_name,&subname);
	    if((S_ISDIR(subname.st_mode)))
	      dir_info->sub_info[curelem-1].sub_dirs++;
	  }
	  chdir(dirname); /* back to Ausgangsverzeichnis */
	}
      } /* else is_dir */
    } /* while n-- */
  } /* else n<0 */
  return(0);
}

/* ****************************************************************** */

char *file_output(const char *name)
{
  FILE *out;

  char *ausgabe;
  char *aufruf;

  if( !(aufruf = (char *) malloc( (sizeof(char)*strlen(name))+40)))
    exit(1);
  if( !(ausgabe = (char *)malloc(sizeof(char)*300) ))
    exit(1);

  sprintf(aufruf,"file %s ",name);
  fflush(NULL); /* flush stdout buffers */
  out=popen(aufruf,"r"); /* opens a pipe(2) and fork(2) */
  fgets(ausgabe,290,out); /* 290 == max stringlen of ausgabe*/

  pclose(out); /* close the filedescriptor */
  return ausgabe;
}

/* ****************************************************************** */


char **ls1(const char *dirname)
{
  struct stat     name;
  struct dirent **namelist;

  int n=0, a=0, i=0;
  char **rueck=NULL;

  chdir(dirname);
  n = scandir(dirname,&namelist,NULL,alphasort);
  if (n<0)
    perror("scandir");
  else
    while(n--)
    {
      stat(namelist[n]->d_name,&name);
      if(S_ISDIR(name.st_mode))
	/* we don't need dirs, just nondirs */
	continue;
      else
      {
	a = strlen(namelist[n]->d_name)+2;
	
	if( !(rueck = (char **) realloc(rueck, sizeof(char *)*(i+1))) )
	  exit(1);
	if( !(rueck[i] = (char *) calloc(a, sizeof(char))) )
	  exit(1);
	strcpy(rueck[i], namelist[n]->d_name);

	i++;
      }
    }
  free(namelist);
  
  return rueck;
}

