/*
 *   3dfb -- a graphical multidimensional filebrowser
 *   written by Tilmann Bitterberg (tilmann@bitterberg.de)
 *   Copyright (C) 1999 by Tilmann Bitterberg
 *   
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *   
 *   Thanks to Max Kliche <max@kliche.org> for support an ideas
 *   Also thanks to the HfT-Stuttgart <http://www.fht-stuttgart.de>
 */

#include "myhdrs.h"

/*
 * TODO: clickable files
 */

/* Global variables */
float matrix[16];             /* temp space for Matrix Mults */
char  workingdir[256];        /* holds current directory, redundant */

GLint xStart = 0, yStart = 0; /* needed for mouse moves */

#ifdef __glut_h__
GLint moverightbutton, movemiddlebutton; /* for mouse events */
GLint moveleftbutton; /* for mouse events */
#endif

GLboolean viewFiles = GL_FALSE;       /* when true, files are displayed */
GLboolean viewHelp  = GL_FALSE;       /* when true, help is displayed */

GLboolean drawsubdirnames  = GL_TRUE; /* when true, names are drawn */

struct dir_info_struct dir_info;      /* this is the global struct from were
				       * DrawScene reads. */  
char ** files_in_dir;                 /* all files in workingdir */

char helpstring[] = {\
"Welcome to 3dfb!\n\
********************************\n\
The multidimensional FileBrowser\n\
\n\
Usage: (like in VI :-)\n\n\
  <SPACE> ............. Display this help\n\
  d ................... Display files\n\
  r ................... Reset viewpoint\n\
  h ................... Rotate left\n\
  l ................... Rotate right\n\
  j ................... Zoom away\n\
  k ................... Zoom into\n\
  u ................... Rotate up\n\
  m ................... Rotate down\n\
  z, y ................ Move up\n\
  n ................... Move down\n\
  Left Button .......... Go to clicked subdir\n\
  Middle Button + Move .. up/down and left/right\n\
  Right Button + Move ... to/away and rotate left/right\n\
  \n\
  <ESC>, q ............ Exit, Quit\n"\
};

/* ****************************************************************** */

int main (int argc, char** argv)
{
  int i=1;
#ifdef __glut_h__
  /*printf("We're using the glut Library.\n");*/
#endif
  /* searching for -V or --version */

  for (;i<argc;i++)
  {
    if(!strcmp(argv[i], "-V") || !strcmp(argv[i], "--version")) {
      fprintf(stdout, "3dfb version %s\n", VERSION);
      fprintf(stdout, 
	  "Visit me on the web at\n"
	  "<http://www.bitterberg.de/tilmann/3dfb.html>\n");
      exit(0);
    }
  }

  if (argc < 2) {
    fprintf(stderr, "You MUST specify a path\n");
    fprintf(stderr, "Example: %s \"/usr\"\n", argv[0]);
    exit (1);
  }
  strcpy(workingdir,argv[1]);
  files_in_dir = ls1( (const char *) workingdir);
  fprintf(stdout, "\n%s", helpstring);
#ifdef __glut_h__
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowSize(730, 730);
  glutInitWindowPosition(540, 200);
  glutCreateWindow(argv[0]);
  glutDisplayFunc(DrawScene);
  glutReshapeFunc(reshape);
  glutMouseFunc(doPicking);
  glutMotionFunc(mouseMove);
  glutKeyboardFunc(keyboard);
  initialize();
  glutMainLoop();
#else
  get_dir_info(workingdir, &dir_info);
  initialize(argv[0]);
  auxMainLoop(DrawScene);
#endif
  return (0);
}

/* ****************************************************************** */

GLvoid DrawScene(GLvoid)
{
  /* clear all pixels */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glEnable(GL_LIGHTING);

  /* when help is shown, disable lighting and names */
  if ((viewHelp == GL_TRUE) || (viewFiles == GL_TRUE)) {
    drawsubdirnames = GL_FALSE;
    glDisable(GL_LIGHT1);
  }
  else {
    drawsubdirnames = GL_TRUE;
    glEnable(GL_LIGHT1);
  }

  /* this prints the working directory at the top of the scene */
  glDisable(GL_LIGHTING);
  glPushMatrix(); /* Save the current Matrix */
    glLoadIdentity();
    STD_TRANS;
    glTranslatef(0.0, 2.0, -0.2);
    glScalef(0.0125, 0.025, 0.0);
    glColor3f(1.0, 0.0, 1.0);
    drawWord(workingdir);
  glPopMatrix(); /* Restore old current Matrix */
  glEnable(GL_LIGHTING);

  /* draws the main scene */
  glPushMatrix(); /* Save the current Matrix */
    glRotatef( -90.0, 10.0, 0.0, 0.0);
    glPushName(dir_info.sub_dirs+1);
      drawprisma(dir_info.sub_dirs);
    glPopName();
    drawText();
  glPopMatrix(); /* Restore old current Matrix */

  if (viewHelp == GL_TRUE)
  {
    glDisable(GL_LIGHTING);
    glPushMatrix(); /* Save the current Matrix */
      glLoadIdentity();
      STD_TRANS;
      glTranslatef(-0.8, 1.7, 0.0);
      glScalef(0.0125, 0.025, 0.0);
      glColor3f(1.0, 1.0, 1.0);
      drawWord(helpstring);
    glPopMatrix(); /* Restore old current Matrix */
    glEnable(GL_LIGHTING);
  }

  if (viewFiles == GL_TRUE)
  {
    glDisable(GL_LIGHTING);
      glColor3f(1.0, 1.0, 1.0);
      drawfiles();
    glEnable(GL_LIGHTING);
  }
  
  /* Swap front and back buffer */
#ifdef __glut_h__
  glutSwapBuffers();
#else
  auxSwapBuffers();
#endif
} /* DrawScene */

/* ****************************************************************** */
#ifdef __glut_h__
GLvoid initialize(GLvoid)
{
  /* set clearing (Background) Color */
  glClearColor(0.0, 0.0, 0.0, 1.0);
  glShadeModel(GL_FLAT);
  glEnable(GL_DEPTH_TEST);
  get_dir_info(workingdir, &dir_info);
  init_light();

} /* initialize */

/* ****************************************************************** */

GLvoid reshape(int w, int h)
{
  /* initialize viewing Values */
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);

  glFrustum(-1.0, 1.0, -1.0, 1.0, 5.0, 1000.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  STD_TRANS;
}

/* ****************************************************************** */
#else /* this is the AUX function */
GLvoid initialize(char *progname)
{
  GLsizei width, height, winSize;
  auxGetScreenSize(&width, &height);

  auxInitPosition(width/4, height/4, width/2, height/2);
  /* tells where to position a window on the screen */
  
  auxInitDisplayMode(AUX_RGBA|AUX_DOUBLE);
  /* tells auxInitWindow to create a RGBA-window */

  auxInitWindow(progname);
  /* opens a window: the string progname appears 
     the title bar */

  auxAnimation(AUX_ON); 

  auxReshapeFunc(resize);
  glClearColor(0.0, 0.0, 0.0, 1.0);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  glViewport(0, 0, width , height);

  /*glOrtho(-2.0, 2.0, -2.0, 2.0, -2.0, 2.0);*/
  glFrustum(-1.0, 1.0, -1.0, 1.0, 5.0, 1000.0);

  glEnable(GL_DEPTH_TEST); /* Tiefenspeicher bereitstellen */
  
  /*gluLookAt(0.3, 0.3, 0.3, -0.3, -0.3, -0.3, 0.0, 0.0, 1.0);*/

  glShadeModel(GL_FLAT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  /* Standart verschiebung nach hinten */
  STD_TRANS;

  init_light();

  /* KEYS */
  auxKeyFunc(AUX_ESCAPE, auxQuit);
  auxKeyFunc(AUX_q, auxQuit);
  auxKeyFunc(AUX_RIGHT, rightkey);
  auxKeyFunc(AUX_l, rightkey);
  auxKeyFunc(AUX_LEFT, leftkey);
  auxKeyFunc(AUX_h, leftkey);
  auxKeyFunc(AUX_UP, upkey);
  auxKeyFunc(AUX_k, upkey);
  auxKeyFunc(AUX_DOWN, downkey);
  auxKeyFunc(AUX_j, downkey);
  auxKeyFunc(AUX_u, rupkey);
  auxKeyFunc(AUX_m, rdownkey);
  auxKeyFunc(AUX_n, tokey);
  auxKeyFunc(AUX_y, awaykey);
  auxKeyFunc(AUX_z, awaykey);
  auxKeyFunc(AUX_r, resetkey);
  auxKeyFunc(' ', viewhelp);
  auxKeyFunc(AUX_d, viewfiles);
  auxMouseFunc( AUX_LEFTBUTTON, AUX_MOUSEDOWN, doPicking);

  /* Register user input functions for producing motion */
  /* This sucks, only one event per button */
  auxMouseFunc(AUX_MIDDLEBUTTON, AUX_MOUSEDOWN, startMove);
  auxMouseFunc(AUX_MIDDLEBUTTON, AUX_MOUSEUP, stopMove);
  auxMouseFunc(AUX_RIGHTBUTTON, AUX_MOUSEDOWN, startMove);
  auxMouseFunc(AUX_RIGHTBUTTON, AUX_MOUSEUP, stopMove);
}
/* **************************************************** */

GLvoid resize(GLsizei width, GLsizei height)
{
  glViewport(0, 0, width, height);
} /* resize */

#endif

/* ****************************************************************** */

GLvoid init_light(GLvoid)
{
  GLfloat light_pos[]={0.0, -1.0, 1.0, 0.0};
  GLfloat light_diff[]={0.7, 0.7, 0.7, 1.0};

  GLfloat lmodel_ambient[]={0.3, 0.3, 0.3, 1.0};

  /* Define Background light */
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);

  /* Double sided lighting */
  glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

  glLightfv(GL_LIGHT1, GL_POSITION, light_pos);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, light_diff);

  glEnable(GL_LIGHT1);
} /* init_light */

/* ****************************************************************** */

/* 
 * the following functions are for registering and processing
 * user keyboard and mouse events when using glut.
 */
#ifdef __glut_h__

GLvoid keyboard(unsigned char key, int x, int y)
{
  char myword[2];
  switch (key) {
    case 27: /* Escape */ exit(0); break;
    case 'q': exit(0); break;
    case 'r': resetkey(); break;
    case 'h': leftkey();  break;
    case 'l': rightkey(); break;
    case 'k': upkey(); break;
    case 'j': downkey(); break;
    case 'u': rupkey(); break;
    case 'm': rdownkey(); break;
    case 'n': tokey(); break;
    case 'z': awaykey(); break;
    case 'y': awaykey(); break;
    case 'd': viewfiles(); break;
    case ' ': viewhelp(); break;
    default: myword[0]=key;
    }

  glutPostRedisplay();
}

/* ****************************************************************** */

GLvoid doPicking(int button, int state, int x, int y)
{
  GLuint selectBuf[BUFSIZE];
  GLint hits;
  GLint mouseX, mouseY;

  GLint viewport[4];

  if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
  {
    /* do the Picking stuff */
    glGetIntegerv (GL_VIEWPORT, viewport);
    mouseX = x;
    mouseY = viewport[3]-y;

    glSelectBuffer (BUFSIZE, selectBuf);

    /* enable GL_SELECT Mode to pick objects */
    (void) glRenderMode(GL_SELECT);

    glInitNames(); /* init Name Stack */
    glPushName(-1); /* throw Name -1 away */

    glMatrixMode(GL_PROJECTION);

    /* Save projection Matrix */
    glGetFloatv(GL_PROJECTION_MATRIX, matrix);
    glPushMatrix(); /* Save the current Matrix */
      glLoadIdentity();
      gluPickMatrix( (GLdouble) mouseX, (GLdouble) mouseY, 
	  5.0, 5.0,  /* x, y pixel tolerance for pick */
	  viewport );
      glMultMatrixf(matrix);
      glMatrixMode( GL_MODELVIEW );
      DrawScene(); /* process DrawScene in GL_SELECT mode */
      hits = glRenderMode(GL_RENDER);
      printHits( hits, selectBuf );
      glMatrixMode( GL_PROJECTION );
    glPopMatrix(); /* Restore old current Matrix */
    glFlush();
    /*printf("Hits = %d, MouseX=%d, MouseY=%d\n", hits, x, y);*/

    glMatrixMode(GL_MODELVIEW);
  }

  /* 
   * this may look a little more complicated than with the
   * aux stuff, but now two or more events can happen at the
   * same time. This means, you can press MIDDLE *and* RIGHT
   * button and move the mouse.
   */

  if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) {
    moveleftbutton = 1;
    /* update global starting positions */
    xStart = x;
    yStart = y;
  }
  if (button == GLUT_RIGHT_BUTTON && state == GLUT_UP)
    moveleftbutton = 0;

  if (button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN) {
    moverightbutton = 1;
    /* update global starting positions */
    xStart = x;
    yStart = y;
  }
  if (GLUT_RIGHT_BUTTON == button && GLUT_UP == state)
    moverightbutton = 0;

  if (GLUT_MIDDLE_BUTTON == button && GLUT_DOWN == state) {
    movemiddlebutton = 1;
    /* update global starting positions */
    xStart = x;
    yStart = y;
  }
  if (button == GLUT_MIDDLE_BUTTON && state == GLUT_UP)
    movemiddlebutton = 0;
    

} /* mouse */

/* ****************************************************************** */

GLvoid mouseMove(int x, int y)
{
  GLint     xOffset, yOffset;

  /* Using the stored and current mouse positions, compute how far
   *    the mouse moved */
  xOffset = x - xStart;
  yOffset = y - yStart;

  /* Update the stored mouse position for later use */
  xStart = x;
  yStart = y;

  if (1 == moverightbutton)
  {
    glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
    glLoadIdentity();
    glTranslatef(0.0, 0.0, yOffset/50.0);
    glMultMatrixf(matrix);

    glRotatef(abs(xOffset/10.0), 0.0, xOffset, 0.0);
  }

  if (1 == movemiddlebutton)
  {
    /*
    glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
    glLoadIdentity();
    glTranslatef(xOffset/200.0, -yOffset/200.0,  0.0);
    glMultMatrixf(matrix);
    */
    glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
    glLoadIdentity();
    STD_TRANS;
    glRotatef(yOffset/50.0, 1.0, 0.0,  0.0);
    STD_TRANS_NEG;
    glMultMatrixf(matrix);
  }

  if (1 == moveleftbutton)
  {
    glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
    glLoadIdentity();
    glTranslatef(xOffset/200.0, -yOffset/200.0,  0.0);
    glMultMatrixf(matrix);
  }

  glutPostRedisplay();
}

#else

/* **************************************************** */
/* now we're not using glut.h */
GLvoid  doPicking( AUX_EVENTREC *event )
{
  GLuint selectBuf[BUFSIZE];
  GLint hits;
  GLint viewport[4];
  GLint mouseX, mouseY;
  GLdouble aspect;

  glGetIntegerv (GL_VIEWPORT, viewport);
  aspect = (GLdouble) viewport[2] / (GLdouble) viewport[3];

  mouseX = event->data[AUX_MOUSEX];
  mouseY = viewport[3] - event->data[AUX_MOUSEY];

  glSelectBuffer (BUFSIZE, selectBuf);
  glRenderMode( GL_SELECT );
  glInitNames();
  glPushName( -1 );

  glMatrixMode(GL_PROJECTION);
  glGetFloatv(GL_PROJECTION_MATRIX, matrix);
  glPushMatrix(); /* Save the current Matrix */
    glLoadIdentity();
    gluPickMatrix( (GLdouble) mouseX, (GLdouble) mouseY, 
      5, 5,  /* x, y pixel tolerance for pick */
	viewport );
    
    glMultMatrixf(matrix);
    glMatrixMode(GL_MODELVIEW);

    DrawScene();

    hits = glRenderMode(GL_RENDER);
    /*printf("Hits = %d, MouseX=%d, MouseY=%d\n", hits, mouseX, mouseY);*/
    printHits( hits, selectBuf );

    glMatrixMode(GL_PROJECTION);
  glPopMatrix(); /* Restore old current Matrix */
  glMatrixMode(GL_MODELVIEW);
}

/* **************************************************** */

/* Define user input functions for mouse events */
GLvoid startMove(AUX_EVENTREC *event)
{
  /* Set up function to process mouse movements */
  auxMouseFunc( AUX_MOUSELOC, NULL, mouseMove );

  /* Record where the mouse pointer was when the user pressed the
   *    left mouse buton */
  xStart = event->data[AUX_MOUSEX];
  yStart = event->data[AUX_MOUSEY];
}

/* **************************************************** */

GLvoid stopMove(AUX_EVENTREC *event)
{
  /* Remove function to process mouse movements */
  auxDeleteMouseFunc(AUX_MOUSELOC, NULL, mouseMove);
}

/* **************************************************** */

GLvoid mouseMove(AUX_EVENTREC *event)
{
  GLint     xPos, yPos;
  GLint     xOffset, yOffset;

  xPos = event->data[AUX_MOUSEX];
  yPos = event->data[AUX_MOUSEY];

  /* Using the stored and current mouse positions, compute how far
   *    the mouse moved */
  xOffset = xPos - xStart;
  yOffset = yPos - yStart;

  /* Update the stored mouse position for later use */
  xStart = xPos;
  yStart = yPos;

  if (AUX_RIGHTBUTTON == event->data[AUX_MOUSESTATUS])
  {
    glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
    glLoadIdentity();
    glTranslatef(0.0, 0.0, yOffset/50.0);
    glMultMatrixf(matrix);

    glRotatef(1, 0.0, xOffset/10.0, 0.0);
  }

  if (AUX_MIDDLEBUTTON == event->data[AUX_MOUSESTATUS])
  {
    glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
    glLoadIdentity();
    glTranslatef(xOffset/200.0, -yOffset/200.0,  0.0);
    glMultMatrixf(matrix);
  }

}
#endif

/* **************************************************** */

/* This function is not good, but it works */
/* FIXME: rewrite it */

/* Processes the hits and change to the clicked directory */
GLvoid printHits (GLint hits, GLuint buffer[])
{
  unsigned int i, j, nameCount;
  GLuint *bufp;

  bufp = (GLuint *) buffer;

  for (i = 0; i < hits; i++) 
  {	/*  for each hit  */
    nameCount = *bufp; 

    bufp++;
    bufp++;
    bufp++;

    for ( j = 0; j < nameCount; j++) 
    {	/*  for each name */
      if (*bufp <= dir_info.sub_dirs) 
      {
	char * temp=NULL;
	/* 
	 * FIXME: clickable files
	 */
	fprintf( stdout, "Changing Dir to:\t%s\n", \
	    dir_info.sub_info[*bufp].sub_dir_name);
	get_dir_info(dir_info.sub_info[*bufp].sub_dir_name, &dir_info);
	strcpy(workingdir,getcwd(temp, 1024));
	/* workingdir is now the full path */
	free(temp);
	files_in_dir = ls1( (const char *) workingdir);
      }
      bufp++;
    }
  }
#ifdef __glut_h__
  glutPostRedisplay();
#endif
}

/* 
 * The following functions are "just" to bind the
 * keys to the correspondending function and to
 * draw the objects and text. 
 */

/* ****************************************************************** */

GLvoid viewhelp(GLvoid) {
  viewHelp = !viewHelp;
}

/* ****************************************************************** */

GLvoid viewfiles(GLvoid) {
  viewFiles = !viewFiles;
}

/* **************************************************** */

GLvoid resetkey(GLvoid){
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(-1.0, 1.0, -1.0, 1.0, 5.0, 1000.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  STD_TRANS;
}

/* **************************************************** */

GLvoid tokey(GLvoid){
  glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
  glLoadIdentity();
  glTranslatef(0.0, 0.05, 0.0);
  glMultMatrixf(matrix);
}

/* **************************************************** */

GLvoid awaykey(GLvoid){
  glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
  glLoadIdentity();
  glTranslatef(0.0, -0.05, 0.0);
  glMultMatrixf(matrix);
}

/* **************************************************** */

GLvoid rupkey(GLvoid) {
  glMatrixMode(GL_PROJECTION);
  STD_TRANS;
  glRotatef(-2, 1.0, 0.0, 0.0);
  STD_TRANS_NEG;
  glMatrixMode(GL_MODELVIEW);
}

/* **************************************************** */

GLvoid rdownkey(GLvoid) {
  glMatrixMode(GL_PROJECTION);
  STD_TRANS;
  glRotatef(2, 1.0, 0.0, 0.0);
  STD_TRANS_NEG;
  glMatrixMode(GL_MODELVIEW);
}

/* **************************************************** */

GLvoid leftkey(GLvoid) {
  glRotatef(2, 0.0, 1.0, 0.0);
}

/* **************************************************** */

GLvoid rightkey(GLvoid) {
  glRotatef(-2, 0.0, 1.0, 0.0);
}

/* **************************************************** */

GLvoid upkey(GLvoid) {
  glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
  glLoadIdentity();
  glTranslatef(0.0, 0.0, 0.5);
  glMultMatrixf(matrix);
}

/* **************************************************** */

GLvoid downkey(GLvoid) {
  glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
  glLoadIdentity();
  glTranslatef(0.0, 0.0, -0.5);
  glMultMatrixf(matrix);
} 

/* ****************************************************************** */

GLvoid drawfiles(GLvoid)
{
  GLfloat angle; /* angle in degree */
  GLfloat angcur; /* current angle in rad */
  GLfloat anghalb; /* half angle in rad */
  int i=0;
  GLint corners = (GLint) dir_info.non_dirs;
  /* get_dir_info returns the right thing */
  
  if (corners<2) corners=2;

  angle=360.0 / ((GLfloat) (corners+1));
  anghalb=RAD(angle/2.0);

  for (i=0; i< corners; i++) 
  {
    glPushMatrix();
      angcur=RAD(angle*i);
  
      glTranslatef(0.0, 1.0, 0.0);
      glTranslatef((sin(anghalb+angcur))*(1.0+((float) 0)), \
	  0.1* (float) i, (cos(anghalb+angcur))*(1.0+((float) 0)) );
      glScalef(0.025, 0.025, 0.0);
  
      drawWord(files_in_dir[i]);
    glPopMatrix();
  }
}

/* **************************************************** */

GLvoid drawText(GLvoid)
{
  GLfloat angle, angle2; /* angle in degree */
  GLfloat sincur; /* current sinus in rad */
  GLfloat coscur; /* current cosinus in rad */
  GLfloat angcur; /* current angle in rad */
  GLfloat anghalb; /* half angle in rad */
  GLfloat sincurhalb, coscurhalb;
  GLfloat anghalb2;
  GLint i=0, f=0;
  GLint corners=dir_info.sub_dirs;
  GLint corners2;

  corners--;

  if (corners<2) corners=2;

  angle=360.0 / ((GLfloat) (corners+1));
  anghalb=RAD(angle/2.0);

  glDisable(GL_LIGHTING); 
  for (i=0; i<=corners; i++) 
  {

    angcur=RAD(angle*(i)); 
    sincur=sin(angcur); 
    coscur=cos(angcur);
    sincurhalb=sin(anghalb+angcur);
    coscurhalb=cos(anghalb+angcur);

    if (GL_TRUE == drawsubdirnames) {
      /* 
       * this draws the subdirectory name and translates, rotates 
       * and scales it correctly. 
       */
      glPushMatrix(); /* Save the current Matrix */
        glTranslatef(sincur, coscur, 0.2);
	glRotatef(-(angle/2.0+angle*i), 0.0, 0.0, 1.0);
	glRotatef(45.0, 0.0, -1.0, 0.0);
	glRotatef(45.0, 1.0, 0.0, 0.0);
	glScalef(0.0125, 0.025, 0.0);
	glColor3f(1.0, 1.0, 1.0);
	drawWord(dir_info.sub_info[i].sub_dir_name);
      glPopMatrix(); /* Restore old current Matrix */
    }

    glPushMatrix(); /* Save the current Matrix */
      glTranslatef(sincurhalb*(2.0+((float) i)), \
	  coscurhalb*(2.0+((float) i)), 0.25*i);
      glRotatef(-(angle/2.0+angle*(i)), 0.0, 0.0, 1.0);
      glRotatef(-90.0+(2.0*i), 1.0, 0.0, 0.0);
      glEnable(GL_LIGHTING); 
      if (i != corners) 
      {
	/* scale the whole thing down */
	glScalef(0.5, 0.5, 0.5);
	/* the Subdir is draw now */
	
	glPushName(i); /* Names from 0 to sub_dirs */
	  drawprisma(dir_info.sub_info[i].sub_dirs);
	glPopName();

	/* draw something (tetrahedrons) around subdirs */

	corners2 = dir_info.sub_info[i].sub_dirs;
	if (corners2<3) corners2=3;
	angle2=360.0 / ((GLfloat) (corners2));
	anghalb2=RAD(angle2/2.0);

	for (f=0; f<=corners2; f++)
	{
	  /* just to save some multiplikations */
	  GLfloat angcur2=RAD(angle2*f); 
	  GLfloat factor2=(1.0+(corners2*0.1));
	  GLfloat sincur2=sin(anghalb2+angcur2)*factor2;
	  GLfloat coscur2=cos(anghalb2+angcur2)*factor2;

	  /* draw the tetrahedrons */
	  if ( f != corners2) {
	    glPushMatrix(); /* Save the current Matrix */

	      glTranslatef(sincur2, coscur2, -0.02);
	      glRotatef(f*10.0, 0.0, -1.0, 1.0);
#ifdef __glut_h__
	      glScalef(0.4, 0.4, 0.4);
	      glutSolidTetrahedron();
#else
	      auxSolidTetrahedron(0.3);
#endif
	    glPopMatrix(); /* Restore old current Matrix */
	  }
	  glDisable(GL_LIGHTING); 
	  /* Draw Lines to tetrahedrons */
	  glPushMatrix(); /* Save the current Matrix */
	    glBegin(GL_LINES);
	      glColor3f(1.0, 1.0, 0.0);
	      glVertex3f(0.0, 0.0, -0.02);
	      glVertex3f(sincur2, coscur2, -0.02);
	    glEnd();
	  glPopMatrix(); /* Restore old current Matrix */
	  glEnable(GL_LIGHTING); 
	} /* for f to corners2 */

      } /* i != corners */
      glDisable(GL_LIGHTING); 
    glPopMatrix(); /* Restore old current Matrix */

    glPushMatrix(); /* Save the current Matrix */
      /* red lines as path to the subdir */
      if (i != corners) {
	glBegin(GL_LINES);
          glColor3f(1.0, 0.0, 0.0);
	  glVertex3f(sincurhalb, coscurhalb, 0.0);
	  glVertex3f(sincurhalb*(2.0+((float) i)), \
	      coscurhalb*(2.0+((float) i)), 0.25*i);
        glEnd();
      } /* if i != corners */
    glPopMatrix(); /* Restore old current Matrix */

  } /* for corners */
	  
  glEnable(GL_LIGHTING); 
} /* draw text */

/* **************************************************** */

GLvoid drawprisma(GLint corners)
{
  GLfloat angle;
  GLfloat angcur, sincur, coscur;
  GLfloat sincurhalb, coscurhalb;
  GLfloat anghalb;
  GLint i=0;
  GLfloat len=0;

  GLfloat mat_ambient[]={0.1, 0.1, 0.1, 1.0};
  GLfloat mat_diffuse[]={1.0, 1.0, 0.0, 1.0};

  GLfloat mat_ambient_back[]={0.00, 0.0, 1.0, 1.0};
  GLfloat mat_diffuse_back[]={1.0, 0.0, 1.0, 1.0};

  GLfloat mat_ambient_pol[]={1.0, 1.0, 1.0, 1.0};
  GLfloat mat_diffuse_pol[]={1.0, 1.0, 1.0, 1.0};

  /* get_dir_info returns the right thing */
  corners--;
  
  if (corners<2) corners=2;

  angle=360.0 / ((GLfloat) (corners+1));
  anghalb=RAD(angle/2.0);

  glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
  
  /* Untersatz */
  glBegin(GL_TRIANGLE_FAN);
    glVertex3f(0.0, 0.0, -1.0);/* Spitze */

    for (i=0; i<= corners+1; i++) 
    {
      angcur=RAD(angle*i);
      sincur=sin(angcur); 
      coscur=cos(angcur);
      sincurhalb=sin(anghalb+angcur);
      coscurhalb=cos(anghalb+angcur);

      /* norm */
      len = sqrt(sincurhalb*sincurhalb + coscurhalb*coscurhalb + 1.0);
      
      glNormal3f(sincurhalb/len, coscurhalb/len, -1.0/len);
      glVertex3f(sincur, coscur, 0.0);
    }
  glEnd();

  /* Kranz */
  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient_back);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse_back);
  glBegin(GL_QUAD_STRIP);
    for (i=0; i<= corners+1; i++) 
    {
      angcur=RAD(angle*i); 
      sincur=sin(angcur); 
      coscur=cos(angcur);
      sincurhalb=sin(anghalb+angcur);
      coscurhalb=cos(anghalb+angcur);

      /* norm */
      len = sqrt(sincurhalb*sincurhalb + coscurhalb*coscurhalb + 1.0);

      glVertex3f(sincur,coscur,0.0);
      glVertex3f(1.1*sincur,1.1*coscur,0.1); /* obere */
      glNormal3f(sincurhalb/len, coscurhalb/len, 1.0/len);
    }
  glEnd();

  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient_pol);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse_pol);
  
  glBegin(GL_POLYGON);
    glNormal3f(0.0, 0.0, 1.0);
    for (i=0; i<= corners; i++) 
    {
      glVertex3f( sin(RAD(angle*i)), cos(RAD(angle*i)), 0.0);
    }
  glEnd();

} /* drawprisma */
